mod collectors;
mod lifecycle;

use axum::{Router, BoxError};
use axum::extract::Path;
use axum::http::StatusCode;
use axum::response::{IntoResponse};
use axum::routing::get;
use axum_tracing_opentelemetry::{response_with_trace_layer, opentelemetry_tracing_layer};
use std::net::SocketAddr;
use collectors::PromMetric;

fn convert_into_metrics(data: Vec<impl PromMetric>) -> String {
    let metrics: Vec<String> = data
        .into_iter()
        .map(|w| w.to_metric_string())
        .collect();

    metrics.join("\n")
}

async fn worlds() -> impl IntoResponse {
    let resp = match collectors::player_count::get_player_count().await {
        Ok(r) => r,
        Err(_) => return (StatusCode::INTERNAL_SERVER_ERROR, "Nope".to_string())
    };

    (StatusCode::OK, convert_into_metrics(resp))
}

async fn stats(Path(rsn): Path<String>) -> impl IntoResponse {
    let resp = match collectors::stats::get_player_stats(&rsn).await {
        Ok(r) => r,
        Err(_) => return (StatusCode::INTERNAL_SERVER_ERROR, "Nope".to_string())
    };

    (StatusCode::OK, convert_into_metrics(resp))
}


#[tokio::main]
async fn main() -> Result<(), BoxError> {
    axum_tracing_opentelemetry::tracing_subscriber_ext::init_subscribers()?;

    let app = Router::new()
        .route("/worlds", get(worlds))
        .route("/stats/:rsn", get(stats))
        .layer(response_with_trace_layer())
        .layer(opentelemetry_tracing_layer());

    let addr = SocketAddr::from(([0, 0, 0, 0], 3030));

    tracing::warn!("listening on {}", addr);
    axum::Server::bind(&"0.0.0.0:3030".parse().unwrap())
        .serve(app.into_make_service())
        .with_graceful_shutdown(lifecycle::shutdown::shutdown_signal())
        .await?;

    Ok(())
}
